/** Class to control the speed of the robot, to get the robot to move, 
    and to display a message about what the robot detects for Assignment P.
    @author Dan Kelly
		@version 1.0
*/

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Hashtable;
import java.io.*;
import java.net.*;

public class ButtonPanel extends JPanel implements ActionListener {

  // Declare and initialize necessary values
	private JButton moveButton;
	private RobotPanel robotPanel;
	private int refreshSpeed = 151;
	private int numberOfObjectsDetected = 0;
	private String message = "Objects Detected : 0";
	private String tempMessage = "Objects Detected : ";

  /** Sets the background color, adds one button to start the animation, 
	    a slider to adjust the speed at which the animation occurs, and a 
			robot detection message.  */
	public ButtonPanel (RobotPanel rp) {
		
    JSlider slider = new JSlider(JSlider.HORIZONTAL,1, 301, 151);
    slider.addChangeListener(new SliderListener());
    slider.setMajorTickSpacing(20);
    slider.setMinorTickSpacing(10);
    slider.setPaintTicks(true);
    Hashtable labelTable = new Hashtable();
    labelTable.put(new Integer(1),new JLabel("Fast"));
    labelTable.put(new Integer(151),new JLabel("Speed"));
    labelTable.put(new Integer(301),new JLabel("Slow"));
    slider.setLabelTable(labelTable);
    slider.setPaintLabels(true);
    slider.setBorder(BorderFactory.createEmptyBorder(0,0,10,0));
    		
    moveButton = new JButton("Move");
    add(moveButton);
    add(slider);
    moveButton.addActionListener(this);
    setBackground(Color.red);
    robotPanel = rp;  
  }

  /** This method is invoked each time
      the panel has to be repainted. You
      should not invoke it directly. */
	public void paintComponent(Graphics g) {
	  super.paintComponent(g);
		g.setColor(Color.black);  // Draw the robot detection message
		g.drawString(message,(4*getWidth())/10,getHeight()/2);
	}

  /** This method is invoked each time the user clicks on the 
	    "Move" button in the GUI.  It must never be called directly. */
	public void actionPerformed (ActionEvent evt) {
    Object source = evt.getSource();
    if(source == moveButton) {
      int j=0,x,y;
      double slope, slopeX, slopeY, deltaX, deltaY;
      boolean control;
			
			// Get mouse click coordinates from the robotPanel
      int moveToX[]=new int[robotPanel.getNumberOfMouseClicks()];
      int moveToY[]=new int[robotPanel.getNumberOfMouseClicks()];
      moveToX = robotPanel.getMoveToX();
      moveToY = robotPanel.getMoveToY();

      // Get the polygons that are on the robotPanel
			int numberOfPolygons = robotPanel.getNumberOfPolygons();
			Polygon[] polyArray=new Polygon[numberOfPolygons]; 
      polyArray = robotPanel.getPolyArray();
			   
		  // Get the number of mouse clicks from the robotPanel
			int numberOfMouseClicks = robotPanel.getNumberOfMouseClicks();
			
			// Get the robot from the robotPanel
      Robot r = robotPanel.getRobot();
			Polygon robotPolygon = r.getRobotPolygon();
			
			// Loop until the robot moves to all mouse click locations
      for(int i=0;i<=(numberOfMouseClicks-1);i++) {
        
				// Do not set the heading of the robot if it hasn't reached 
				// the first mouse click location yet.  It has been previously set.
				if(i != 0)
          r.setHeading(moveToX[i]-r.getCenterOfRobotX(),-moveToY[i]+r.getCenterOfRobotY());
                                
        // Calculate the distance the robot needs to travel in both the x and y directions, 
				// and the slope of the line of movement 
        deltaX = moveToX[i] - r.getCenterOfRobotX();
        deltaY = moveToY[i] - r.getCenterOfRobotY();
				slope = (deltaY / deltaX);  
				slopeX = slope;
				slopeY = slope;
				
			  // Set necessary values for movement in each quadrant
				if ((deltaX > 0) && (deltaY > 0)) {  // First Quadrant
				 	x = 1;
				  y = 1;
			  }
				else if ((deltaX < 0) && (deltaY > 0)) {  // Second Quadrant
				  x = -1;
				  y = 1;
				  slopeX = -slope;
				}
				else if ((deltaX < 0) && (deltaY < 0)) {  // Third Quadrant
				  x = -1;
				  y = -1;
				  slopeX = -slope;
				  slopeY = -slope;
				}
				else if ((deltaX > 0) && (deltaY < 0)){  // Fourth Quadrant
				  x = 1;
				  y = -1;
				  slopeY = -slope;
				}
				else if ((deltaX == 0) && (deltaY > 0)) { // Positive Y-Axis
				  x = 0;
					y = 1;
				}
				else if ((deltaX == 0) && (deltaY < 0)) { // Negative Y-Axis
				  x = 0;
					y = -1;
				}
				else if ((deltaX > 0) && (deltaY == 0)) { // Positive X-Axis
				  x = 1;
					y = 0;
				}
				else {                                    // Negative X-Axis
				  x = 0;
					y = -1;
				}
				
				
			  // Set a boolean to control how the move method in Robot is executed.
				if(Math.abs(deltaX) > Math.abs(deltaY))
				  control = true;
				else
				  control = false;
                                  
        // Loop until the robot reaches the mouse click location
        while((j <= Math.abs(deltaX)) || (j <= Math.abs(deltaY)))  {

          // Sleep for 100 milliseconds
          try { 

            // Move the robot
            r.move(slopeX,slopeY,x,y,control);
                                       
            // Tell the robotPanel to go paint itself.
	          robotPanel.paintMe();
												
						for(int k=0;k<numberOfPolygons;k++) {
						  if(r.inSensorRange(polyArray[k])) {
							  	numberOfObjectsDetected++;
	            } // END if
						} // END for

				    message = tempMessage+""+numberOfObjectsDetected+"";
						
						// Reset the number of objects detected.
						numberOfObjectsDetected = 0;
						
            // paint buttonPanel
						paintImmediately(0,0,getWidth(),getHeight());
	    
			      // Sleep for the user-determined amount of time
						Thread.currentThread().sleep(refreshSpeed);
          } catch (InterruptedException e) {} 
            
				  // Increment while loop control variable
					j++;
        } // END while 
				
				// Reset while loop control variable
				j = 0;            
      } // END for
			
			// Reset numberOfMouseClicks to Zero, and reset a flag value
      robotPanel.resetNumberOfMouseClicks();
		  robotPanel.resetFlag();
    } // END if moveButton
		
  } // END actionPerformed
	
	/** Listens to the slider. */
  class SliderListener implements ChangeListener {
    /** Sets the speed at which the animation takes place */
		public void stateChanged(ChangeEvent e) {
      JSlider source = (JSlider)e.getSource();
        if (source.getValueIsAdjusting()) { 
          refreshSpeed = (int)source.getValue();
        }      
    } // END stateChanged
  } // END SliderListener
		
} // END ButtonPanel
