/** 	The abstract class <tt>ObjectClass</tt> holds information used
	to create the polygons produced in @see <tt>RobotPanel</tt>.
	It is extended by the classes 	@see <tt>RectangleObject</tt>
					@see <tt>TriangleObject</tt>
					@see <tt>OvalObject</tt>
					@see <tt>FirstObject</tt>
					@see <tt>SecondObject</tt>
					@see <tt>ThirdObject</tt>

	@author Carey Fry
	@version 1.0 
*/

import javax.swing.*;
import java.awt.*;

public abstract class ObjectClass {

	protected static String smSize = "Small";    	// string indicating small size
   	protected static int  	smWidth = 25, 		// width of sm-sized object
   				smHeight = 25;		// height of sm-sized object	
	
  	protected static String medSize = "Medium"; 	// string indicating medium size	
  	protected static int 	medWidth =  50, 	// width of med-sized object	
   				medHeight = 50;		// height of med-sized object	
	
 	protected static String lgSize = "Large"; 	// string indicating large size		
  	protected static int 	lgWidth = 100, 		// width of lg-sized object	
  				lgHeight = 100;		// height of lg-sized object		
	
	protected static String pinkString = "Pink", 	// string indicating pink object	
				greenString = "Green", 	// string indicating green object	
				orangeString = "Orange";  // string indicating orange object	

 	protected static final Color 	pink = Color.pink,                     
					green = Color.green,  
					orange = Color.orange;

	protected int 	width,			// width of object
 			height, 		// height of object	
			X, 			// X coordinate of top left
			Y,			// Y coordinate of top left		
			minX = 10,		// minimum X		
			maxX = 600 - 10,	// maximum X		
			minY = 10,		// minimum Y
			maxY = 300 - 10;	// maximum Y

    protected Color objectColor;		// final color of object

    protected Polygon p = new Polygon();  	// polygon containing object



/** 	The method <tt>ObjectClass</tt> with no parameters 
	creates an instance of the superclass <tt>Object</tt>.

 */

   	 public ObjectClass() {
		super();
	} // END constructor

/** 	The method <tt>ObjectClass</tt> is a constructor 
	that creates an instance with the size and color
	specified in the parameters and a random location.

	@param	size	size of the object; <tt>small</tt>, 
			<tt>medium</tt>, or <tt>large</tt>.
	@param 	color	color of the object; <tt>pink</tt>,
			<tt>green</tt>, or <tt>orange</tt>.
	*/

	public ObjectClass(String size, String color) {
		
		if (size.equals(smSize)){		// set small size
			width = smWidth;
	                height = smHeight;
	        } else if (size.equals(medSize)) {	// set medium size
    	 		width = medWidth;
	                height = medHeight;
	       } else if (size.equals(lgSize)) {	// set large size
			width = lgWidth;
			height = lgHeight;
		} else { 	// error message in case of invalid size parameter					System.out.println("Size invalid: medium by default.");
			width = medWidth;		// set medium size
	   		height = medHeight;	        
		} // END if

		// set random location
		X = (int)(Math.random()*((maxX - width) - minX) + minX);
		Y = (int)(Math.random()*((maxY - height) - minY) + minY);

	
	if (color.equals(pinkString))		// set pink color
		objectColor = pink;
	else if (color.equals(greenString))	// set green color
		objectColor = green;
	else if (color.equals(orangeString))	// set orange color
		objectColor = orange;
	else {	// error message in case of invalid color parameter
		System.out.println("Color invalid: pink by default.");
		objectColor = pink;
		} // END if
	} // END constructor

/** 	The method <tt>GetObject</tt> returns the polygon
	that represents the instance of <tt>ObjectClass</tt>.

	@return	 the polygon representing the object*/

	public Polygon GetObject() {
		return(p);	
		} // END GetObject
} // END class