/** Panel in which all the objects, the robot, and 

    the robot's path are painted for Assignment P.

    @author Dan Kelly

		@version 1.0

*/



import javax.swing.*;

import java.awt.*;

import java.awt.event.*;



public class RobotPanel extends JPanel {



  // Contants for the maximum number of mouse clicks and polygons allowed

  private final int MAX_NUMBER_OF_MOUSE_CLICKS = 25;

	private final int MAX_NUMBER_OF_POLYGONS = 5;

	// Declare and initialize the number of mouse clicks and polygons

	private int numberOfMouseClicks = 0;

	private int numberOfPolygons = 0;

	// Declare array to hold the obstacle polygons and an array to hold their colors

  private Polygon[] polyArray = new Polygon[MAX_NUMBER_OF_POLYGONS];

	private Color[] colorArray = new Color[MAX_NUMBER_OF_POLYGONS];

	// Declare arrays to hold the mouse click locationss 

	private int moveToX[] = new int[MAX_NUMBER_OF_MOUSE_CLICKS];

	private int moveToY[] = new int[MAX_NUMBER_OF_MOUSE_CLICKS];

	// Declare a new Robot

	private Robot r=new Robot();

	// Declare polygons for the robot and its sensor

	private Polygon robotPolygon, sensorPolygon;

	// Declare and initialize temporary variables to hold the center of the robot

	private int tempX, tempY;

	// Declare "Move" button

	private JButton moveButton;

	// Declare a flag value

	private boolean flag = true;

	

	/** Create a panel to draw the robot, it's path, and obstacles.

      It is a MouseListener.

			*/

	public RobotPanel () {

		

	  // Set temporary variables

		tempX = r.getCenterOfRobotX();

		tempY = r.getCenterOfRobotY();

	

	  // Set the initial heading

		moveToX[0] = tempX+1;

		moveToY[0] = tempY;

    r.setHeading(moveToX[numberOfMouseClicks]-r.getCenterOfRobotX(),-moveToY[numberOfMouseClicks]+r.getCenterOfRobotY());

		

		// Make the panel a MouseListener

		addMouseListener(new MouseAdapter() {

      public void mouseClicked(MouseEvent evt) {

			  if(numberOfMouseClicks <= MAX_NUMBER_OF_MOUSE_CLICKS) {

				  // Store location of mouse click into an array

          moveToX[numberOfMouseClicks] = evt.getX();

          moveToY[numberOfMouseClicks] = evt.getY();

				  // Set heading of the first mouse click

				  if(numberOfMouseClicks == 0) {

				    r.setHeading(moveToX[0]-r.getCenterOfRobotX(),-moveToY[0]+r.getCenterOfRobotY());

          } // END if

				 	repaint();

				  // Keep running total of mouse clicks

			    numberOfMouseClicks++;	

				} // END if

			} // END mouseClicked

		}); // END addMouseListener

		

		// Set the background color

    setBackground(Color.black);



  } // END Constructor



  /** This method is invoked each time

      the panel has to be repainted. You

      should not invoke it directly. */

  public void paintComponent (Graphics g) {



    super.paintComponent(g);



    // Get the Robot and Sensor polygons

		robotPolygon = r.getRobotPolygon();

    sensorPolygon = r.getSensorPolygon();

		

		// Draw the polygons to the panel

		g.setColor(Color.blue);

    g.fillPolygon(sensorPolygon);    

    

		// Draw Obstacles

		for(int i=0;i<numberOfPolygons;i++) {

		  g.setColor(colorArray[i]);

		  g.fillPolygon(polyArray[i]);

		}

		

		// Draw line from Center of robot to the first mouse click

		if((numberOfMouseClicks > 0) && flag) {

		  tempX = r.getCenterOfRobotX();

			tempY = r.getCenterOfRobotY();

		  g.setColor(Color.red);

		  g.drawLine(tempX,tempY,moveToX[0],moveToY[0]);

			flag = false;

		} //END if

		else if ((numberOfMouseClicks > 0) && !flag) {

		  g.setColor(Color.red);

		  g.drawLine(tempX,tempY,moveToX[0],moveToY[0]);

		}

		

		// Draw Path

		if(numberOfMouseClicks > 0) {

		  g.setColor(Color.red);

		  for(int i=0;i<(numberOfMouseClicks-1);i++)

		    g.drawLine(moveToX[i],moveToY[i],moveToX[i+1],moveToY[i+1]);

		} //END if

		

		// Draw the robot

		g.setColor(Color.yellow);

    g.fillPolygon(robotPolygon);

        

    // Draw a line from the center of the robot to the front of the robot

		g.setColor(Color.black);

		g.drawLine(r.getCenterOfRobotX(),r.getCenterOfRobotY(),r.getFrontOfRobotX(),r.getFrontOfRobotY());

		

  } // END paintComponent

	

  /** Get the number of times the user has clicked on the panel

	    @return The number of times the user has clicked on the panel 

			*/

	public int getNumberOfMouseClicks () {

    return (numberOfMouseClicks);

  }



  /** Get the robot

	    @return The instance of the robot

			*/

  public Robot getRobot() {

    return r;

  }

  

	/** Get the array of the X coordinates of the locations where the user clicked on the panel

      @return An array of X coordinates

			*/

  public int[] getMoveToX () {

    return moveToX;

  }

  

	/** Get the array of the Y coordinates of the locations where the user clicked on the panel.

      @return An array of Y coordinates

			*/

  public int[] getMoveToY () {

    return moveToY;

  }

  

	/** Reset the number of times the user has clicked on the panel to zero. */

  public void resetNumberOfMouseClicks () {

    numberOfMouseClicks = 0;

  }

	

	/** Reset the the flag that controls the first line of the robot's path to <t>true</t>. */

  public void resetFlag () {

    flag = true;

  }

	

  /** Repaint the panel immediately. */

	public void paintMe () {

    paintImmediately(0,0,getWidth(),getHeight());

  }

	

	/** Get the array of polygons that have been painted on the panel.

      @return An array of polygons

			*/

  public Polygon[] getPolyArray () {

    return polyArray;

  }



  /** Get the number of polygons that have been painted on the panel.

	    @return An integer representing the number of polygons painted on the panel

			*/

	public int getNumberOfPolygons () {

    return numberOfPolygons;

  }

	

	/** Reset the number of polygons to zero and repaint the panel. */

	public void resetNumberOfPolygons() {

	  numberOfPolygons = 0;

		paintMe();

	}

	

	/** Insert the <i>Polygon</i> <t>p</t>, of the <i>Color</i> represented by the 

	    <i>String</i> <t>s</t>, into an array, and repaint the panel. 

			@param p The <i>Polygon</i> to be inserted into the array

			@param s The <i>String</i> that represents the <i>Color</i> of <t>p</t>

			*/

	public void setArrays (Polygon p, String s) {

	  // Insert the polygon and its color if the number of polygons on the panel is 

		// less than the maximum

		if(numberOfPolygons <= MAX_NUMBER_OF_POLYGONS) {

		  // Insert polygon

			polyArray[numberOfPolygons] = p;

		  // Set the polygon's color

			if(s == "Pink")

		    colorArray[numberOfPolygons] = Color.pink;

		  else if(s == "Green")

		    colorArray[numberOfPolygons] = Color.green;

		  else

		    colorArray[numberOfPolygons] = Color.orange;

		  // Increment the number of polygons

			numberOfPolygons++;

		  // Repaint the panel

			paintMe();

		} // END if

	} // END setArrays



} // END RobotPanel

